from re import template
import re
from jinja2 import Template, defaults
import jinja2
import json
from .translation_dictionary import translation
import os


templates = {   "default": "template.py", 
                "spark:3.2.0":"spark_3_2_0.py", 
                "spark:3.2.1":"spark_3_2_0.py"}

# Load jinja template
def get_template(path):
    path = os.path.join(os.path.dirname(os.path.realpath(__file__)), path)
    templateLoader = jinja2.FileSystemLoader(os.path.dirname(path))
    templateEnv = jinja2.Environment(loader=templateLoader)
    return templateEnv.get_template(name=os.path.basename(path))



def generate_meta_model(name, description, framework, area, mode, properties, input_datasets, output_datasets, input_models, output_models, docker_image, json=json):
    
    template = get_template(os.path.join("templates", templates[framework]))

    # If using SCDF, name must be shorter than 30 chars
    assert(len(name)<=30), f"Service name must be shorter than 30 chars, got: {name} with {len(name)} chars"
    
    # If using SCDF, name must be shorter than 35 chars
    assert(re.match('^[a-z0-9]([-a-z0-9]*[a-z0-9])?$', name)), f"Service name must start with a letter and must not contain special characters except \"-\". Spaces will be converted to dashes. Got: {name} instead. Good example: \"my-service-name-10\""


    outputText = template.render(name=name, description=description, area = area, properties=properties, json=json, translation=translation, 
                                input_datasets=input_datasets, output_datasets=output_datasets,
                                input_models=input_models, output_models=output_models,
                                docker_image=docker_image,
                                mode=mode
                                )

    return outputText




{
    "name": "{{ name }}",
    "description": "{{ description }}",
    "mode": "{{mode.upper()}}",
    "area": "{{area.upper()}}",
    "url":  {{json.dumps(docker_image)}},
    "version": "1.0.0",
    "framework": {
        "id": 6,
        "name": "Python",
        "version": "3",
        "imageUrl": "https://cdn.alidalab.it/static/images/frameworks/python_logo.png"
    },
    "assets": { 
        "datasets": 
            {"input":[
                {% for input_dataset in input_datasets %}
                {   "name":{{json.dumps(input_dataset.name)}},
                    "description": {{json.dumps(input_dataset.description)}},
                    "type": "tabular",
                    "col_type": {{json.dumps(translation['column_types'][input_dataset.columns_type])}},
                    "order": {{loop.index-1}}
                },
                {% endfor %}
                ],
            "output": [
                {% for output_dataset in output_datasets %}
                {   "name":{{json.dumps(output_dataset.name)}},
                    "description": {{json.dumps(output_dataset.description)}},
                    "type": "tabular",
                    "order": {{loop.index-1}}
                },
                {% endfor %}
                ]
            }
    },
    "properties": [
        {% for property in properties %}
        {
            "it.eng.alidalab.applicationcatalogue.domain.service.ApplicationProperty": {
                "description": {{json.dumps(property.description)}},
                "mandatory": {{json.dumps(property.required)}},
                "defaultValue": {{json.dumps(property.default)}},
                "value": null,
                "key": {{json.dumps(property.name)}},
                "type": {{json.dumps(translation['type'][property.type])}},
                "inputData": null,
                "outputData": null
            }
        },
        {% endfor %}
        {% for output_dataset in output_datasets %}
        {
            "it.eng.alidalab.applicationcatalogue.domain.service.ApplicationProperty": {
                "defaultValue": null,
                "description": {{json.dumps(output_dataset.description)}},
                "key": "output-dataset",
                "mandatory": true,
                "type": "STRING",
                "value": null,
                "inputData": null,
                "outputData": true
            }
        },
        {% endfor %}

        {% if input_models|length + output_models|length != 0  %}
        {
            "it.eng.alidalab.applicationcatalogue.domain.service.StaticProperty": {
                "description": "The URL for Web HDFS service",
                "mandatory": true,
                "defaultValue": null,
                "value": null,
                "key": "webHdfsUrl",
                "type": "STRING"
            }
        },
        {% endif %}

        {% for input_model in input_models %}
        {
            "it.eng.alidalab.applicationcatalogue.domain.service.StaticProperty": {
                "description": "The storage type where the input model is stored.",
                "mandatory": true,
                "defaultValue": null,
                "value": "hdfs",
                "key": "dataStorageType-input-model",
                "type": "STRING"
            }
        },
        {
            "it.eng.alidalab.applicationcatalogue.domain.service.ApplicationProperty": {
                "description": "HDFS path where to read the model",
                "mandatory": true,
                "defaultValue": null,
                "value": null,
                "key": "input-model",
                "type": "STRING",
                "inputData": true,
                "outputData": false
            }
        },
        {% endfor %}

        {% for output_model in output_models %}
        {
            "it.eng.alidalab.applicationcatalogue.domain.service.StaticProperty": {
                "description": "The storage type where the input model is stored.",
                "mandatory": true,
                "defaultValue": null,
                "value": "hdfs",
                "key": "dataStorageType-output-model",
                "type": "STRING"
            }
        },
        {
            "it.eng.alidalab.applicationcatalogue.domain.service.ApplicationProperty": {
                "description": "HDFS path where to read the model",
                "mandatory": true,
                "defaultValue": null,
                "value": null,
                "key": "output-model",
                "type": "STRING",
                "inputData": true,
                "outputData": false
            }
        },
        {% endfor %}

    ],
    "metrics": []
}

